VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "StringBuilder"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'-----------------------------------------------------------------------------
' Utilities from http://blog.nkadesign.com/microsoft-access/
' (c) Renaud Bompuis, 2013
' Licensed under the Creative Commons Attribution License
' http://creativecommons.org/licenses/by/3.0/
' http://creativecommons.org/licenses/by/3.0/legalcode
'
' Free for re-use in any application or tutorial providing clear credit
' is made about the origin of the code and a link to the site above
' is prominently displayed where end-user can access it.
'-----------------------------------------------------------------------------
Option Compare Database
Option Explicit

'=============================================================================
' Internal fields
'=============================================================================

'-----------------------------------------------------------------------------
' The .Net m_StrBuilder object we'll use internally to build the string$
'-----------------------------------------------------------------------------
Private m_StrBuilder As Object
Private fAlwaysAppendCrLf As Boolean

'=============================================================================
' Initialise / Terminate
'=============================================================================

Private Sub Class_Initialize()
    Set m_StrBuilder = CreateObject("System.Text.StringBuilder")
End Sub

Private Sub Class_Terminate()
    Set m_StrBuilder = Nothing
End Sub

'=============================================================================
' Public properties
'=============================================================================

'-----------------------------------------------------------------------------
' Get or Set the current capacity of this instance.
' This is nornally automatically managed by the StringBuilder but can be set
' manually if neccessary, for instance for better performance on very large
' strings being built of a large amount of small ones.
'-----------------------------------------------------------------------------
Public Property Get Capacity() As Long
    Capacity = m_StrBuilder.Capacity
End Property

Public Property Let Capacity(value As Long)
    If value < Length Then Exit Property
    m_StrBuilder.Capacity = value
End Property

'-----------------------------------------------------------------------------
' Get or Set the Length of this instance.
' This is the actual length of the resulting string, not to be confused with its Capacity.
' Setting the Length to less than its current value will truncate this instance.
'-----------------------------------------------------------------------------
Public Property Get Length() As Long
    Length = m_StrBuilder.Length
End Property

Public Property Let Length(value As Long)
    If value < 0 Then
        m_StrBuilder.Length = 0
    Else
        m_StrBuilder.Length = value
    End If
End Property

'-----------------------------------------------------------------------------
' Whether calling Append() always adds a CRLF at the end of the string being added.
' Default: False
'-----------------------------------------------------------------------------
Public Property Get AlwaysAppendCrLf() As Boolean
    AlwaysAppendCrLf = fAlwaysAppendCrLf
End Property

Public Property Let AlwaysAppendCrLf(value As Boolean)
    fAlwaysAppendCrLf = value
End Property

'-----------------------------------------------------------------------------
' Obtain the character at the given index in this instance
' index starts at 0.
'-----------------------------------------------------------------------------
Public Function Chars(index As Long) As String
    If index < 0 Or index >= Length Then Exit Function
    Chars = ChrW$(m_StrBuilder.Chars(index))
End Function

'-----------------------------------------------------------------------------
' Obtain the codepoint value of the character at the given index in this instance
' index starts at 0.
'-----------------------------------------------------------------------------
Public Function Codes(index As Long) As Long
    If index < 0 Or index >= Length Then Exit Function
    Codes = m_StrBuilder.Chars(index)
End Function


'=============================================================================
' Public Methods
'=============================================================================

'-----------------------------------------------------------------------------
' Append a value to this instance. If there are parameters passed, the value is
' interpreted as a format string and will be interpolated before being appended.
' @defaultMember
'-----------------------------------------------------------------------------
Public Function Append(ByRef value As Variant, ParamArray args() As Variant) As StringBuilder
Attribute Append.VB_UserMemId = 0
    Set Append = Me
    
    Dim v As Variant
    v = ConvertVariantToBaseType(value)
    
    Select Case VarType(v)
        Case vbString
            If UBound(args) = -1 Then
                ' No parameters given, just append as a string
                m_StrBuilder.Append_3 UnEscStr(v)
            Else
                ' Unpack args if they were passed verbatim from another ParamArray
                Dim params() As Variant
                If (UBound(args) - LBound(args) = 0) And (VarType(args(0)) = (vbArray + vbVariant)) Then
                    params = args(0)
                Else
                    params = args
                End If
                ' Double check that all the parameters are of a type that we can handle otherwise convert to base type
                Dim i As Long
                Dim p As Variant
                For i = LBound(params) To UBound(params)
                    p = ConvertVariantToBaseType(params(i))
                    If VarType(p) = VbVarType.vbString Then params(p) = UnEscStr(p) Else params(p) = p
                Next i
                m_StrBuilder.AppendFormat_4 UnEscStr(v), params
            End If
        Case vbDate
            m_StrBuilder.AppendFormat "{0}", v
        Case vbBoolean
            m_StrBuilder.Append_5 v
        Case vbInteger
            m_StrBuilder.Append_9 v
        Case vbLong
            m_StrBuilder.Append_10 v
        #If Win64 Then
        Case vbLongLong
            m_StrBuilder.Append_17 v
        #End If
        Case vbSingle
            m_StrBuilder.Append_12 v
        Case vbDouble
            m_StrBuilder.Append_13 v
        Case vbDecimal
            m_StrBuilder.Append_14 v
        Case vbCurrency
            m_StrBuilder.Append_14 v
        Case vbByte
            m_StrBuilder.Append_7 v
        ' Other types are ignored
        Case vbEmpty
        Case vbNull
        Case Else
            m_StrBuilder.Append_3 sFormat("{0}", v)
        
    End Select
    
    If AlwaysAppendCrLf Then m_StrBuilder.Append_3 vbCrLf
    
End Function

'-----------------------------------------------------------------------------
' Append a value and terminates it with a CRLF.
' @param value is the value to append to the current instance.
' If the value is not a string, it will be converted to a string in using
' its default convertions format.
' @returns this instance.
'-----------------------------------------------------------------------------
Public Function AppendLine(Optional ByRef value As Variant) As StringBuilder
    Set AppendLine = Me
    Append value
    m_StrBuilder.Append_3 vbCrLf
End Function

'-----------------------------------------------------------------------------
' Inserts the given string in this instance at the specified index.
' If there are additional parameters passed, the string is interpreted as
' a format string and will be interpolated before being inserted.
' @param index the location in the string, starting from the first character
'              at index 0, where the value will be inserted.
' @param value is the value to be inserted. It will be converted to a string
'              using the type's default conversion rules before being inserted.
'              For instance, a Date will be converted to teh default date format
'              specified for the
'-----------------------------------------------------------------------------
Public Function Insert(index As Long, ByRef value As Variant, ParamArray args() As Variant) As StringBuilder
    Set Insert = Me
    Dim v As Variant
    v = ConvertVariantToBaseType(value)
    Select Case VarType(v)
        Case vbString
            If UBound(args) = -1 Then
                m_StrBuilder.Insert_2 index, UnEscStr(v)
            Else
                ' Unpack args if they were passed verbatim from another ParamArray
                Dim params() As Variant
                If (UBound(args) - LBound(args) = 0) And (VarType(args(0)) = (vbArray + vbVariant)) Then
                    params = args(0)
                Else
                    params = args
                End If
                m_StrBuilder.Insert_2 index, sFormat(CStr(value), params)
            End If
        Case vbDate
            m_StrBuilder.Insert index, sFormat("{0}", v)
        Case vbBoolean
            m_StrBuilder.Insert_3 index, v
        Case vbInteger
            m_StrBuilder.Insert_6 index, v
        Case vbLong
            m_StrBuilder.Insert_10 index, v
        #If Win64 Then
        Case vbLongLong
            m_StrBuilder.Insert_11 index, v
        #End If
        Case vbSingle
            m_StrBuilder.Insert_12 index, v
        Case vbDouble
            m_StrBuilder.Insert_13 index, v
        Case vbDecimal
            m_StrBuilder.Insert_14 index, v
        Case vbCurrency
            m_StrBuilder.Insert_14 index, v
        Case vbByte
            m_StrBuilder.Insert_5 index, v
        ' Other types are ignored
        Case vbEmpty
        Case vbNull
        Case Else
            m_StrBuilder.Insert index, sFormat("{0}", v)
            
    End Select
    
End Function

'-----------------------------------------------------------------------------
' Replace a section of this instance given by oldValue by new content given by newValue.
' Optionally give a start index to start the search from and a maximum replacement count.
'-----------------------------------------------------------------------------
Public Function Replace(ByRef oldValue As String, ByRef newValue As String, Optional startIndex As Long = -1, Optional count As Long = -1) As StringBuilder
    Set Replace = Me
    If startIndex < 0 Or count < 0 Then
        m_StrBuilder.Replace oldValue, UnEscStr(newValue)
    Else
        m_StrBuilder.Replace_2 oldValue, UnEscStr(newValue), startIndex, count
    End If
End Function

'-----------------------------------------------------------------------------
' Removes the specified range of characters from this instance.
'-----------------------------------------------------------------------------
Public Function Remove(startIndex As Long, Length As Long) As StringBuilder
    Set Remove = Me
    m_StrBuilder.Remove startIndex, Length
End Function

'-----------------------------------------------------------------------------
' Returns the string representation of this instance.
' If there are parameters passed, this instance is interpreted as
' a format string and will be interpolated before being returned.
'-----------------------------------------------------------------------------
Public Function ToString(ParamArray args() As Variant) As String
    If UBound(args) = -1 Then
        ToString = m_StrBuilder.ToString()
    Else
        ' Unpack args if they were passed verbatim from another ParamArray
        Dim params() As Variant
        If (UBound(args) - LBound(args) = 0) And (VarType(args(0)) = (vbArray + vbVariant)) Then
            params = args(0)
        Else
            params = args
        End If
        ToString = sFormat(m_StrBuilder.ToString(), params)
    End If
End Function

'-----------------------------------------------------------------------------
' Returns a substring of this instance starting from the character at startIndex
' and of given length.
' If length is omitted, all characters until the end of the string will be returned.
'-----------------------------------------------------------------------------
Public Function ToSubString(startIndex As Long, Optional Length As Long = -1) As String
    If Length = -1 Then
        Length = m_StrBuilder.Length - startIndex
        If Length <= 0 Then Exit Function
    End If
    ToSubString = m_StrBuilder.ToString_2(startIndex, Length)
End Function

'-----------------------------------------------------------------------------
' Returns True if the given StringBuilder instance have equal string
' as this instance.
'-----------------------------------------------------------------------------
Public Function Equal(sb As StringBuilder) As Boolean
    Equal = m_StrBuilder.Equal(sb)
End Function

'-----------------------------------------------------------------------------
' Empty the string.
' Same as setting length = 0.
'-----------------------------------------------------------------------------
Public Sub Clear()
    Length = 0
End Sub


